<?php

namespace Drupal\breadcrumb_overrides\Breadcrumb;

use Drupal\Core\Breadcrumb\Breadcrumb;
use Drupal\Core\Breadcrumb\BreadcrumbBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;

/**
 * Provides custom breadcrumb builder.
 */
class BreadcrumbOverrideBuilder implements BreadcrumbBuilderInterface {
  use StringTranslationTrait;

  public function __construct(
    TranslationInterface $stringTranslation,
    protected AdminContext $adminContext,
  ) {
    $this->setStringTranslation($stringTranslation);
  }

  /**
   * {@inheritdoc}
   */
  public function applies(RouteMatchInterface $routeMatch) {
    // For now don't apply overrides on the admin routes.
    if ($this->adminContext->isAdminRoute()) {
      return FALSE;
    }
    // Apply this breadcrumb builder only o routes that have a breadcrumb override.
    $breadcrumbOverrides = $this->getBreadcrumbOverride($routeMatch);
    return $breadcrumbOverrides !== NULL && ($breadcrumbOverrides['override'] || $breadcrumbOverrides['disable']);
  }

  /**
   * {@inheritdoc}
   */
  public function build(RouteMatchInterface $routeMatch) {
    $breadcrumb = new Breadcrumb();

    $breadcrumbOverrides = $this->getBreadcrumbOverride($routeMatch);
    if (
      $breadcrumbOverrides === NULL
      || $breadcrumbOverrides['disable']
    ) {
      return $breadcrumb;
    }

    if ($breadcrumbOverrides['override']) {
      foreach ($breadcrumbOverrides['links'] as $link) {
        try {
          $url = Url::fromUri($link['uri']);
        }
        catch (\Exception $e) {
          $url = Url::fromRoute('<none>');
        }
        $breadcrumb->addLink(Link::fromTextAndUrl(
          $link['title'],
          $url,
        ));
      }
    }

    $breadcrumb->addCacheContexts(['route']);
    return $breadcrumb;
  }

  /**
   * Get breadcrumb override for the current route match.
   */
  protected function getBreadcrumbOverride(RouteMatchInterface $routeMatch): ?array {
    $override = [
      'disable' => FALSE,
      'override' => FALSE,
      'links' => [],
    ];

    $node = $routeMatch->getParameter('node');
    if (
      !$node instanceof NodeInterface
      || !$node->hasField('breadcrumb_overrides_links')
      || !$node->hasField('breadcrumb_overrides_disable')
      || !$node->hasField('breadcrumb_overrides_override')
    ) {
      return NULL;
    }

    if (!$node->get('breadcrumb_overrides_disable')->isEmpty()) {
      $override['disable'] = \boolval($node->get('breadcrumb_overrides_disable')->getString());
    }

    if (!$node->get('breadcrumb_overrides_override')->isEmpty()) {
      $override['override'] = \boolval($node->get('breadcrumb_overrides_override')->getString());
    }

    if (!$node->get('breadcrumb_overrides_links')->isEmpty()) {
      $values = $node->get('breadcrumb_overrides_links')->getValue();
      foreach ($values as $key => $value) {
        // Make sure each link has at least a title.
        if (
          !isset($value['title'])
          || !is_string($value['title'])
          || strlen($value['title']) === 0
        ) {
          unset($values[$key]);
        }
      }
      $override['links'] = $values;
    }

    return $override;
  }

}
